local Otis = LibStub("AceAddon-3.0"):NewAddon("Otis", "AceEvent-3.0", "AceHook-3.0", "AceTimer-3.0")

local function GetEncounters(t,k)
	local info = { ByIndex = {}, ByID = {} }

	local index = 1
	local name, _, encounterID = EJ_GetEncounterInfoByIndex(index)
	while encounterID do
		-- print(index, name, encounterID)
		info.ByIndex[index] = encounterID
		info.ByID[encounterID] = index

		index = index + 1
		name, _, encounterID = EJ_GetEncounterInfoByIndex(index)
	end
	rawset(t, k, info)
	return info
end

local tblInstances = setmetatable( {}, { __index = GetEncounters } )
local tblSpecTex = {}
local btnLootSpec, btnUp, btnDown
local idPlayerClass
local dbbosses
local fWaitingOnBonusLoot = false

-- calling this with nil or 0 as the spec ID should hide all icons
local function ShowSpec(specID)
	for id, tex in pairs(tblSpecTex) do
		if id == specID then
			tex:Show()
		else
			tex:Hide()
		end
	end
end

function Otis:UpdateLootSpecButton()
	local instanceID, encounterID = EncounterJournal.instanceID, EncounterJournal.encounterID
	-- print(instanceID, encounterID)
	if encounterID then
		ShowSpec(self.db.char.encounters[encounterID])
		btnLootSpec:Enable()
	else
		ShowSpec(nil)
		btnLootSpec:Disable()
	end
end
	
function btnLootSpec_Click(self, button, down)
	local instanceID, encounterID = EncounterJournal.instanceID, EncounterJournal.encounterID

	if button == "LeftButton" then
		if encounterID then -- save the spec pref
			local classID, specID = EJ_GetLootFilter()
			
			if idPlayerClass ~= classID then
				print("ERROR: wrong class selected")
			elseif specID == 0 then
				print("ERROR: no spec selected")
			else
				-- change encounters table
				Otis.db.char.encounters[encounterID] = specID

				-- change bosses table
				local index = 1
				local id, name = EJ_GetCreatureInfo(index)
				while name do
					Otis.db.char.bosses[name] = specID
					index = index + 1
					id, name = EJ_GetCreatureInfo(index)
				end

				-- change button overlay icon
				ShowSpec(specID)
			end
		end
	elseif button == "RightButton" then
		if encounterID then -- clear setting for this encounter
			-- clear encounters table
			Otis.db.char.encounters[encounterID] = nil

			-- clear bosses table
			local index = 1
			local id, name = EJ_GetCreatureInfo(index)
			while name do
				Otis.db.char.bosses[name] = nil
				index = index + 1
				id, name = EJ_GetCreatureInfo(index)
			end

			-- hide button overlay icon
			ShowSpec(nil)
		end
	end
end

function btnUp_Click()
	if EncounterJournal then
		local instanceID, encounterID = EncounterJournal.instanceID, EncounterJournal.encounterID
		if instanceID and encounterID then
			local info = tblInstances[instanceID]
			local index = info.ByID[encounterID]
			if index > 1 then
				EncounterJournal_DisplayEncounter(info.ByIndex[index-1], true)
			elseif index == 1 then
				EncounterJournal_DisplayInstance(instanceID, true)
			end
		end
	end
end

function btnDown_Click()
	if EncounterJournal then
		local instanceID, encounterID = EncounterJournal.instanceID, EncounterJournal.encounterID
		if instanceID then
			local info = tblInstances[instanceID]
			if encounterID then
				local index = info.ByID[encounterID]
				if index < #info.ByIndex then
					EncounterJournal_DisplayEncounter(info.ByIndex[index+1], true)
				end
			elseif #info.ByIndex > 0 then
				EncounterJournal_DisplayEncounter(info.ByIndex[1], true)
			end
		end
	end
end

function Otis:MakeButtons()
	if (type(EncounterJournalEncounterFrameInfoLootTab) ~= "table") then
		-- print("parent frame doesn't exist yet")
		return
	end

	-- print("making buttons")
	self:UnregisterEvent("EJ_LOOT_DATA_RECIEVED")
	self:UnregisterEvent("EJ_DIFFICULTY_UPDATE")

	btnLootSpec = CreateFrame("Button", "OtisButtonLootSpec", EncounterJournalEncounterFrameInfoModelTab)
	btnLootSpec:SetScript("OnClick", btnLootSpec_Click)
	btnLootSpec:RegisterForClicks("LeftButtonUp", "RightButtonUp")

	btnLootSpec:SetPoint("TOP", EncounterJournalEncounterFrameInfoModelTab, "BOTTOM", 0, 2)
	btnLootSpec:SetSize(63, 57)
	btnLootSpec:SetNormalTexture("Interface\\EncounterJournal\\UI-EncounterJournalTextures")
	btnLootSpec:GetNormalTexture():SetTexCoord(0.25585938, 0.37890625, 0.90332031, 0.95898438)	-- UI-EJ-Tab-Selected
	btnLootSpec:SetDisabledTexture("Interface\\EncounterJournal\\UI-EncounterJournalTextures")
	btnLootSpec:GetDisabledTexture():SetTexCoord(0.25585938, 0.37890625, 0.90332031, 0.95898438)	-- UI-EJ-Tab-UnSelected
	btnLootSpec:GetDisabledTexture():SetDesaturated(true)
	btnLootSpec:SetHighlightTexture("Interface\\EncounterJournal\\UI-EncounterJournalTextures")
	btnLootSpec:GetHighlightTexture():SetTexCoord(0.00195313, 0.12500000, 0.90332031, 0.95898438)	-- UI-EJ-Tab-Highlight

	idPlayerClass = select(3, UnitClass("player"))
	for i=1, GetNumSpecializationsForClassID(idPlayerClass) do
		local id, name, _, icon = GetSpecializationInfo(i)
		local tex = btnLootSpec:CreateTexture(name, "OVERLAY")
		tex:SetTexture(icon)
		tex:SetPoint("BOTTOMLEFT", 20, 13)
		tex:SetPoint("TOPRIGHT", -11, -12)
		tex:Hide()
		tblSpecTex[id] = tex
	end

	btnLootSpec:Disable()

 	self:SecureHook("EncounterJournal_ListInstances", "UpdateLootSpecButton")
	self:SecureHook("EncounterJournal_DisplayInstance", "UpdateLootSpecButton")
	self:SecureHook("EncounterJournal_DisplayEncounter", "UpdateLootSpecButton")

	btnUp = CreateFrame("Button", "OtisButtonUp", EncounterJournalEncounterFrameInfoModelTab)
	btnUp:SetScript("OnClick", btnUp_Click)
--	btnUp:SetPoint("TOP", EncounterJournalEncounterFrameInfoModelTab, "BOTTOM", 6, 2)
	btnUp:SetPoint("TOP", btnLootSpec, "BOTTOM", 6, 2)
	btnUp:SetSize(63, 57)
	btnUp:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollUp-Up")
	btnUp:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollUp-Down")
	btnUp:SetDisabledTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollUp-Disabled")
	btnUp:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")

	btnDown = CreateFrame("Button", "OtisButtonDown", EncounterJournalEncounterFrameInfoModelTab)
	btnDown:SetScript("OnClick", btnDown_Click)
	btnDown:SetPoint("TOP", btnUp, "BOTTOM", 0, 2)
	btnDown:SetSize(63, 57)
	btnDown:SetNormalTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Up")
	btnDown:SetPushedTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Down")
	btnDown:SetDisabledTexture("Interface\\ChatFrame\\UI-ChatIcon-ScrollDown-Disabled")
	btnDown:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight", "ADD")


--[==[
	btnUp:SetNormalTexture("Interface\\MainMenuBar\\UI-MainMenu-ScrollUpButton-Up")
	btnUp:SetPushedTexture("Interface\\MainMenuBar\\UI-MainMenu-ScrollUpButton-Down")
	btnUp:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Disabled")
	btnUp:SetHighlightTexture("Interface\\MainMenuBar\\UI-MainMenu-ScrollUpButton-Highlight", "ADD")
--]==]
end


function Otis:LoadSavedSettings()
	-- saved settings and default values
	local defaults = {
		char = {
			encounters = {},
			bosses = {},
		},
	}

	self.db = LibStub("AceDB-3.0"):New("OtisDB", defaults, true) --, _G.DEFAULT)

	-- we should be able to migrate saved data formats immediately
	local DBVERSION = 1
	
	if self.db.char.version == DBVERSION then
		return
	end
	
	if self.db.char.version == nil then
		self.db.char.version = DBVERSION
		return
	end

	self.db.char.version = DBVERSION
end


-- Called when the addon is enabled
function Otis:OnEnable()
	self:LoadSavedSettings()
	self:RestoreLootSpec()  -- in case you DC in combat

	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")

	self:SecureHookScript(BonusRollFrame, "OnShow", "WaitForBonusLoot")
	self:SecureHookScript(BonusRollFrame, "OnHide", "RestoreLootSpec")
	self:RegisterEvent("MISSING_OUT_ON_LOOT", "RestoreLootSpec")

	self:RegisterEvent("EJ_LOOT_DATA_RECIEVED", "MakeButtons")
	self:RegisterEvent("EJ_DIFFICULTY_UPDATE",  "MakeButtons")
end

-- Called when the addon is disabled
function Otis:OnDisable()
	self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")

	self:UnregisterEvent("EJ_LOOT_DATA_RECIEVED")
	self:UnregisterEvent("EJ_DIFFICULTY_UPDATE ")
	self:UnhookAll()
end

function Otis:WaitForBonusLoot()
	fWaitingOnBonusLoot = true
	self:CancelAllTimers()
end

function Otis:RestoreLootSpec()
	fWaitingOnBonusLoot = false
	if self.db.char.lootSpec then
		-- print(format("Otis restored loot spec to %i", self.db.char.lootSpec))
		SetLootSpecialization(self.db.char.lootSpec)
		self.db.char.lootSpec = nil
	end
end

function Otis:PLAYER_REGEN_DISABLED() -- entered combat
	dbbosses = self.db.char.bosses
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function Otis:PLAYER_REGEN_ENABLED() -- left combat
	self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	if not fWaitingOnBonusLoot then
		-- print("scheduling a restore")
		self:ScheduleTimer("RestoreLootSpec", 5)
	end
end

function Otis:COMBAT_LOG_EVENT_UNFILTERED(_, _, _, _, _, srcName, _, _, _, dstName)
	local specID = dbbosses[srcName] or dbbosses[dstName]
	if specID then
		-- ding ding - we have a saved spec choice for this boss
		self.db.char.lootSpec = GetLootSpecialization()
		SetLootSpecialization(specID)
		-- print(format("Otis changed loot spec from %i to %i", self.db.char.lootSpec, specID))

		-- we've identified this encounter, don't need to process anymore combat log notifications
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end
